#include "PID.h"
#include <string>
#include <vector>
#include <Wire.h>
#include "Debug.h"
#include "LIDAR.h"
#include "Camera.h"
#include <Arduino.h>
#include "Constants.h"
#include "utilities.h"
#include "Propulsion.h"
#include "ColorDetect.h"
#include "Localization.h"
// #include "Adafruit_VL53L0X.h"
using namespace std;

//Identify all the variables that will be used by the robot
int findcolor = 1; //based on the same code from openMV. Determine the color you want to find
char newPTCL = '1';
int pauseState = 0;
int displayTracking = 0;

int ballCapture = 0;
int ballDetect  = 0;
int goalFind    = 0;

float altitude  = 0;

// Function Declarations


//Create the interface that will be used by the camera
openmv::rpc_scratch_buffer<256> scratch_buffer; // All RPC objects share this buffer.
openmv::rpc_i2c_master interface(0x12, 100000); //to make this more robust, consider making address and rate as constructor argument

Camera cam(&interface);

void setup() {
  Serial.begin(BAUD_RATE);
  Wire.begin();
  AFMS.begin();  // create with the default frequency 1.6KHz
  interface.begin(); //communication between ESP and OpenMV
    
  ColorDetect_Setup(findcolor);
  Debug_Setup();
  PID_Setup();
  LIDAR_Setup();
  Localization_Setup();
  #ifdef MESH_H
    Mesh_Setup();
  #endif
}

void loop(){
  Debug_LoopStart();
  #ifdef MESH_H
    MeshUpdate();
  #endif
  
  /* --------- PAUSE STATE --------- */
  while(pauseState == 1){
    Serial.println("pause");
    #ifdef MESH_H
      Mesh_PauseState();
    #endif
    Debug_PauseState();
    Propulsion_PauseState();
  }
  /* ------- END PAUSE STATE ------- */
  
  int id = -1;
  int  x = 0,  y = 0;
  int tx = 0, ty = 0, tz = 0;
  int rx = 0, ry = 0, rz = 0;
  int8_t goal[3] = {0,1,2};
  
  /* -------- STANDBY STATE -------- */
  Debug_StandbyState();
  /* ------ END STANDBY STATE ------ */

  altitude = Localization_Init();
  
  /* --------- INTERRUPTS ---------- */
  altitudeBarrier(altitude);
  
  if(insideWindyArea()){
    // leave windy area
    // flip horizontal inputs
  }
  /* ------ END OF INTERRUPTS ------ */

  /* ---------- MAIN LOOP ---------- */
  // Have we captured the ball?
  ballCapture = LIDAR_BallCapture();
  ballDetect  = cam.exe_color_detection_biggestblob(threshold[0], threshold[1], threshold[2], threshold[3], threshold[4], threshold[5], x, y);
  goalFind    = cam.exe_goalfinder(goal[0], goal[1], goal[2], id, tx, ty, tz, rx, ry, rz);
  
  if(ballCapture){ // Ball captured
	  if(goalFind){ // Ball captured and Goal found
      Debug_GoalFound();
      PID_GoalFound(tx/1.00, ty/1.00);
      Localization_GoalFound(altitude);
	    Propulsion_GoalFound();
    }
    else { // Ball captured but Goal not found
      Debug_GoalSeeking();
      Propulsion_GoalSeeking();
    }
  }
  else { // Ball not captured
	  if(ballDetect) { // Ball not captured but detected
      Debug_BallDetected();
      Localization_BallDetected();
      PID_BallDetected(x/1.00, y/1.00);
      Propulsion_BallDetected();
    }
    else { // Ball not captured and not detected
      Propulsion_BallSeeking();
    }
  }

  // -------- END MAIN LOOP -------- //
}

void altitudeBarrier(int altitude){
  if(altitude < lowHeightBarrier){
    moveVertical((lowHeightBarrier - altitude)*255/lowHeightBarrier);
  }
  else if(altitude > ceilingHeight - highHeightBarrier){
    moveVertical(((ceilingHeight - highHeightBarrier) - altitude)*255/highHeightBarrier);
  }	
}

float insideWindyArea(){
  return 0;
}
