#ifndef PROPULSION_H
#define PROPULSION_H

#include "Constants.h"
#include <Adafruit_MotorShield.h>

int zero = 0;

// Function Declarations
void moveVertical(int vel);
void moveHorizontal(int vel_hori, int base_speed);

// Create the motor shield object with the default I2C address
Adafruit_MotorShield AFMS = Adafruit_MotorShield(); 
Adafruit_DCMotor *motor_VL = AFMS.getMotor(2); 
Adafruit_DCMotor *motor_VR = AFMS.getMotor(1);
Adafruit_DCMotor *motor_HL = AFMS.getMotor(3);
Adafruit_DCMotor *motor_HR = AFMS.getMotor(4);

void Propulsion_PauseState(){
  moveVertical(zero);
  moveHorizontal(zero,zero);
}

void Propulsion_GoalFound(){
  moveVertical(Outputy);
  moveHorizontal(Outputx, BASE_SPEED);
}

void Propulsion_GoalSeeking(){
  moveVertical(0);
  moveHorizontal(SEEKING_SPEED, 0);
}

void Propulsion_BallDetected(){
  moveVertical(Outputy);
  moveHorizontal(Outputx, BASE_SPEED);
}

void Propulsion_BallSeeking(){
  moveVertical(0);
  moveHorizontal(SEEKING_SPEED, 0);
}

//vel value should be between -255 to 255 with positive values moving the blimp upward.
void moveVertical(int vel){
    if (vel > 0) { //up
      panel.singleLED(DEBUG_VERTICALSPEED, abs(vel), 0, 0);
      motor_VL->setSpeed(abs((int) vel));
      motor_VR->setSpeed(abs((int) vel)); 
      motor_VL->run(BACKWARD);
      motor_VR->run(BACKWARD);
    } else if(vel < 0) { //down
      panel.singleLED(DEBUG_VERTICALSPEED, 0, 0, abs(vel));
      motor_VL->setSpeed(abs((int) Outputy));  
      motor_VR->setSpeed(abs((int) Outputy)); 
      motor_VL->run(FORWARD);
      motor_VR->run(FORWARD);
    } else {
      panel.singleLED(DEBUG_VERTICALSPEED, 0, 0, 0);
      motor_VL->setSpeed(0);
      motor_VR->setSpeed(0);
    }
}

void moveHorizontal(int vel_hori, int base_speed){
  int lspeed = vel_hori + base_speed;
  int rspeed = vel_hori + base_speed;

  if (lspeed > 0){
    motor_HL->run(FORWARD);
  } else {
    motor_HL->run(BACKWARD);
  }

  if (rspeed > 0){
    motor_HR->run(FORWARD);
  } else {
    motor_HR->run(BACKWARD);
  }
  displaySpeed(lspeed, rspeed);
  motor_HL->setSpeed(min(MAX_SPEED, abs(rspeed)));
  motor_HR->setSpeed(min(MAX_SPEED, abs(lspeed)));
}

#endif
